/*****************************************************************************
*   Presentation: DBA498 - Extended Events Deep Dive
*   FileName: Demo 7 - Defining_Predicates.sql 
*
*   Summary: Create Event Session to demonstrate defining predicates to
*            filter event firing and short circuiting
*
*   Date: October 7, 2011 
*
*   SQL Server Versions:
*         2008, 2008R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/


--Predicate Sources

	-- State Data Predicates
	SELECT p.name AS package_name,
	        o.name AS source_name,
	        o.description
	FROM sys.dm_xe_objects AS o
	JOIN sys.dm_xe_packages AS p
	      ON o.package_guid = p.guid
	WHERE (p.capabilities IS NULL OR p.capabilities & 1 = 0)
	   AND (o.capabilities IS NULL OR o.capabilities & 1 = 0)
	   AND o.object_type = 'pred_source'
	
--Predicate Comparators

	-- Comparison Predicates
	SELECT p.name AS package_name,
	        o.name AS source_name,
	        o.description
	FROM sys.dm_xe_objects AS o
	JOIN sys.dm_xe_packages AS p
	      ON o.package_guid = p.guid
	WHERE (p.capabilities IS NULL OR p.capabilities & 1 = 0)
	   AND (o.capabilities IS NULL OR o.capabilities & 1 = 0)
	   AND o.object_type = 'pred_compare'


/*************** Looking at the system_health session ***************/
-- C:\Program Files\Microsoft SQL Server\MSSQL10_50.MSSQLSERVER\MSSQL\Install\u_tables.sql


-- The predicates in this session have been carefully crafted to minimize impact of event collection
-- Changing the predicate definition may impact system performance
--
CREATE EVENT SESSION system_health ON SERVER

/* The sql_text and session_id for any sessions that encounter
   an error that has a severity >=20.
   The sql_text and session_id for any sessions that encounter 
   a memory-related error. The errors include 17803, 701, 802, 
   8645, 8651, 8657 and 8902. */
   
ADD EVENT sqlserver.error_reported
(
    ACTION (package0.callstack, sqlserver.session_id, sqlserver.sql_text, sqlserver.tsql_stack)
    -- Get callstack, SPID, and query for all high severity errors ( above sev 20 )
    WHERE severity >= 20
    -- Get callstack, SPID, and query for OOM errors ( 17803 , 701 , 802 , 8645 , 8651 , 8657 , 8902 )
    OR (ERROR = 17803 OR ERROR = 701 OR ERROR = 802 OR ERROR = 8645 OR ERROR = 8651 OR ERROR = 8657 OR ERROR = 8902)
),

/* A record of any non-yielding scheduler problems. (These 
   appear in the SQL Server error log as error 17883.) */
   
ADD EVENT sqlos.scheduler_monitor_non_yielding_ring_buffer_recorded,

/* Any deadlocks that are detected. */

ADD EVENT sqlserver.xml_deadlock_report,

/* The callstack, sql_text, and session_id for any sessions
   that have waited on latches (or other interesting resources)
   for > 15 seconds. */

ADD EVENT sqlos.wait_info
(
    ACTION (package0.callstack, sqlserver.session_id, sqlserver.sql_text)
    WHERE 
    (duration > 15000 AND 
        (    
            (wait_type > 31    -- Waits for latches and important wait resources (not locks ) that have exceeded 15 seconds. 
                AND
                (
                    (wait_type > 47 AND wait_type < 54)
                    OR wait_type < 38
                    OR (wait_type > 63 AND wait_type < 70)
                    OR (wait_type > 96 AND wait_type < 100)
                    OR (wait_type = 107)
                    OR (wait_type = 113)
                    OR (wait_type > 174 AND wait_type < 179)
                    OR (wait_type = 186)
                    OR (wait_type = 207)
                    OR (wait_type = 269)
                    OR (wait_type = 283)
                    OR (wait_type = 284)
                )
            )
            
/* The callstack, sql_text, and session_id for any sessions 
   that have waited on locks for > 30 seconds. */
   
            OR 
            (duration > 30000        -- Waits for locks that have exceeded 30 secs.
                AND wait_type < 22
            ) 
        )
    )
),

/* The callstack, sql_text, and session_id for any sessions 
   that have waited for a long time for preemptive waits. 
   The duration varies by wait type. A preemptive wait is 
   where SQL Server is waiting for external API calls. */
   
ADD EVENT sqlos.wait_info_external
(
    ACTION (package0.callstack, sqlserver.session_id, sqlserver.sql_text)
    WHERE 
    (duration > 5000 AND
        (   
            (    -- Login related preemptive waits that have exceeded 5 seconds.
                (wait_type > 365 AND wait_type < 372)
                OR    (wait_type > 372 AND wait_type < 377)
                OR    (wait_type > 377 AND wait_type < 383)
                OR    (wait_type > 420 AND wait_type < 424)
                OR    (wait_type > 426 AND wait_type < 432)
                OR    (wait_type > 432 AND wait_type < 435)
            )
            OR 
            (duration > 45000     -- Preemptive OS waits that have exceeded 45 seconds. 
                AND 
                (    
                    (wait_type > 382 AND wait_type < 386)
                    OR    (wait_type > 423 AND wait_type < 427)
                    OR    (wait_type > 434 AND wait_type < 437)
                    OR    (wait_type > 442 AND wait_type < 451)
                    OR    (wait_type > 451 AND wait_type < 473)
                    OR    (wait_type > 484 AND wait_type < 499)
                    OR wait_type = 365
                    OR wait_type = 372
                    OR wait_type = 377
                    OR wait_type = 387
                    OR wait_type = 432
                    OR wait_type = 502
                )
            )
        )
    )
)

/* Capture Event information using the ring_buffer target. */

ADD target package0.ring_buffer        -- Store events in the ring buffer target
    (SET max_memory = 4096)

/* Set the session to start automatically with SQL Server */

WITH (startup_state = ON)
GO

-- Look at the Event columns
SELECT
	name,
	column_id,
	type_name
FROM sys.dm_xe_object_columns
WHERE object_name = 'wait_info'
  AND column_type = 'data';

--  Lookup the map values for the mapped types
SELECT *
FROM sys.dm_xe_map_values
WHERE name = 'wait_types'
  AND
    (map_key > 31    -- Waits for latches and important wait resources (not locks ) that have exceeded 15 seconds. 
        AND
        (
            (map_key > 47 AND map_key < 54)
            OR map_key < 38
            OR (map_key > 63 AND map_key < 70)
            OR (map_key > 96 AND map_key < 100)
            OR (map_key = 107)
            OR (map_key = 113)
            OR (map_key > 174 AND map_key < 179)
            OR (map_key = 186)
            OR (map_key = 207)
            OR (map_key = 269)
            OR (map_key = 283)
            OR (map_key = 284)
        ));